package org.fouram.service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.fouram.core.base.service.BaseService;
import org.fouram.core.plugin.weixin.cp.util.WXCpOaUtil;
import org.fouram.core.util.BeanUtils;
import org.fouram.core.util.DateUtil;
import org.fouram.core.util.Tools;
import org.fouram.entity.SapCheck;
import org.fouram.entity.SapCheckOption;
import org.fouram.entity.SapCheckReport;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

import me.chanjar.weixin.cp.bean.oa.WxCpCheckinData;
import me.chanjar.weixin.cp.bean.oa.WxCpCheckinOption;
import me.chanjar.weixin.cp.bean.oa.WxCpCheckinOption.CheckinTime;

@Service
public class SapCheckService extends BaseService {

	@SuppressWarnings("unchecked")
	public List<String> selectCheckUserList() throws Exception {
		return (List<String>) findList("SapCheckMapper.selectCheckUserList", null);
	}

	public Map<String, List<String>> groupList(List<String> list) {
		int listSize = list.size();
		int toIndex = 100;
		Map<String, List<String>> map = new HashMap<>(); // 用map存起来新的分组后数据
		int keyToken = 0;
		for (int i = 0; i < list.size(); i += 100) {
			if (i + 100 > listSize) { // 作用为toIndex最后没有100条数据则剩余几条newList中就装几条
				toIndex = listSize - i;
			}
			List<String> newList = list.subList(i, i + toIndex);
			List<String> userIds = Lists.newArrayList();
			for (String userId : newList) {
				userIds.add(userId);
			}
			map.put("keyName" + keyToken, userIds);
			keyToken++;
		}

		return map;
	}

	public void saveAllUserCheckData(Date startDate, Date endDate) throws Exception {
		// 清空表记录
		update("SapCheckMapper.truncateCheckin", null);
		update("SapCheckMapper.truncateCheckoff", null);
		update("SapCheckMapper.truncateCheckout", null);

		List<String> userIds = selectCheckUserList();
		Map<String, List<String>> map = groupList(userIds);
		int count = 0;
		for (String key : map.keySet()) {
			count = count + 1;
			List<WxCpCheckinData> checkinDatas = WXCpOaUtil.getCheckinData(startDate, endDate, map.get(key));
			System.out.println(checkinDatas.size());
			int subCount = 0;
			for (WxCpCheckinData data : checkinDatas) {
				subCount = subCount + 1;
				SapCheck check = new SapCheck();
				BeanUtils.copyProperties(data, check);
				check.setCheckinDate(new Date(check.getCheckinTime() * 1000));
				check.setCheckinDay(DateUtil.getDay(check.getCheckinDate()));
				if (data.getCheckinType().equals("上班打卡")) {
					saveCheckin(check);
				} else if (data.getCheckinType().equals("下班打卡")) {
					saveCheckoff(check);
				} else if (data.getCheckinType().equals("外出打卡")) {
					saveCheckout(check);
				} else {
					System.out.println(data);
				}
				System.out.println(subCount);
			}
			System.out.println(count);
		}
	}

	public void saveCheckoutStartAndEndData() throws Exception {
		update("SapCheckMapper.truncateCheckoutStart", null);
		update("SapCheckMapper.truncateCheckoutEnd", null);
		update("SapCheckMapper.saveCheckoutStart1", null);
		update("SapCheckMapper.saveCheckoutStart2", null);
		update("SapCheckMapper.saveCheckoutEnd", null);
	}

	public void saveAllUserCheckOption() throws Exception {
		update("SapCheckMapper.truncateCheckOption", null);

		List<String> users = selectCheckUserList();
		Map<String, List<String>> map = groupList(users);
		int count = 0;
		List<Date> datetimes = Lists.newArrayList();
		datetimes.add(DateUtil.sdfTime.parse("2020-05-07 00:00:00"));
		for (String key : map.keySet()) {
			count = count + 1;
			List<WxCpCheckinOption> options = WXCpOaUtil.getCheckinOption(datetimes, map.get(key));
			System.out.println(options.size());
			int subCount = 0;
			for (WxCpCheckinOption option : options) {
				subCount = subCount + 1;
				if (option.getGroup() != null && !option.getGroup().getCheckinDate().isEmpty()) {
					SapCheckOption check = new SapCheckOption();
					CheckinTime checkinTime = option.getGroup().getCheckinDate().get(0).getCheckinTime()[0];
					check.setUserId(option.getUserId());
					check.setWorkSec(checkinTime.getWorkSec());
					check.setOffWorkSec(checkinTime.getOffWorkSec());
					saveCheckinOption(check);
				}
				System.out.println(subCount);
			}
			System.out.println(count);
		}
	}

	public static void main(String[] args) {
		System.out.println(34200 / 60 / 60.0);
		System.out.println(66600 / 60 / 60.0);
	}

	public void saveCheckin(SapCheck check) throws Exception {
		save("SapCheckMapper.saveCheckin", check);
	}

	public void saveCheckoff(SapCheck check) throws Exception {
		save("SapCheckMapper.saveCheckoff", check);
	}

	public void saveCheckout(SapCheck check) throws Exception {
		save("SapCheckMapper.saveCheckout", check);
	}

	public void saveCheckinOption(SapCheckOption check) throws Exception {
		save("SapCheckMapper.saveCheckinOption", check);
	}

	@SuppressWarnings("unchecked")
	public void updateReportBaseData() throws Exception {
		// 处理重复数据
		List<SapCheck> sapChecks = (List<SapCheck>) findList("SapCheckMapper.selectRepeatCheckin", null);
		Set<String> checkinDays = Sets.newHashSet();
		for (SapCheck sapCheck : sapChecks) {
			if (!checkinDays.contains(sapCheck.getCheckinDay() + sapCheck.getUserId())) {
				checkinDays.add(sapCheck.getCheckinDay() + sapCheck.getUserId());
			} else {
				update("SapCheckMapper.deleteCheckin", sapCheck);
			}
		}

		sapChecks = (List<SapCheck>) findList("SapCheckMapper.selectRepeatCheckoff", null);
		checkinDays = Sets.newHashSet();
		for (SapCheck sapCheck : sapChecks) {
			if (!checkinDays.contains(sapCheck.getCheckinDay() + sapCheck.getUserId())) {
				checkinDays.add(sapCheck.getCheckinDay() + sapCheck.getUserId());
			} else {
				update("SapCheckMapper.deleteCheckoff", sapCheck);
			}
		}

		sapChecks = (List<SapCheck>) findList("SapCheckMapper.selectRepeatCheckoutStart", null);
		checkinDays = Sets.newHashSet();
		for (SapCheck sapCheck : sapChecks) {
			if (!checkinDays.contains(sapCheck.getCheckinDay() + sapCheck.getUserId())) {
				checkinDays.add(sapCheck.getCheckinDay() + sapCheck.getUserId());
			} else {
				update("SapCheckMapper.deleteCheckoutStart", sapCheck);
			}
		}

		sapChecks = (List<SapCheck>) findList("SapCheckMapper.selectRepeatCheckoutEnd", null);
		checkinDays = Sets.newHashSet();
		for (SapCheck sapCheck : sapChecks) {
			if (!checkinDays.contains(sapCheck.getCheckinDay() + sapCheck.getUserId())) {
				checkinDays.add(sapCheck.getCheckinDay() + sapCheck.getUserId());
			} else {
				update("SapCheckMapper.deleteCheckoutEnd", sapCheck);
			}
		}
	}

	@SuppressWarnings("unchecked")
	public void saveReportResultData() throws Exception {
		// 生成报表数据
		update("SapCheckMapper.truncateCheckReport", null);
		update("SapCheckMapper.saveCheckReport", null);
		// 更新华人时代人员上班时间可以推迟30分钟
		update("SapCheckMapper.updateHrsdWorkTime", null);
		// 更新当前未打卡情况，未更新微信端应该是晚上才更新的
		update("SapCheckMapper.updateCheckinException", null);
		update("SapCheckMapper.updateCheckoffException", null);
		// 更新所有异常事件的数据为异常数据
		update("SapCheckMapper.updateAllExceptionResult", null);
		// 查询有请假情况的数据
		List<SapCheckReport> checkReports = (List<SapCheckReport>) findList("SapCheckMapper.selectLeaveDayList", null);
		for (SapCheckReport checkReport : checkReports) {
			if (Tools.isInteger(checkReport.getLeaveDay())) {
				update("SapCheckMapper.updateResultOk", checkReport.getId());
			} else {
				// 0.5
				if (isOkWorkTime(checkReport) || isOkOffWorkTime(checkReport)) {
					update("SapCheckMapper.updateResultOk", checkReport.getId());
				}
			}
		}

//		checkReports = (List<SapCheckReport>) findList("SapCheckMapper.selectCheckoutList", null);
		checkReports = (List<SapCheckReport>) findList("SapCheckMapper.selectExceptionList", null);
		for (SapCheckReport checkReport : checkReports) {
			// TODO 由于刚开始使用微信打开，所以特殊处理下，上下班只要一个正常就算正常或者有补卡
			if (isOkWorkTime(checkReport) || isOkOffWorkTime(checkReport) || StringUtils.isNotBlank(checkReport.getBukaDay())) {
				update("SapCheckMapper.updateResultOk", checkReport.getId());
			}
		}
		// 生成报表用户数据
		update("SapCheckMapper.dropCheckReportUser", null);
		update("SapCheckMapper.saveCheckReportUser", null);
	}

	// 是不是正确的上班时间
	private boolean isOkWorkTime(SapCheckReport checkReport) throws Exception {
		if (StringUtils.isBlank(checkReport.getCheckinException())) {
			return true;
		}
		if (StringUtils.isNotBlank(checkReport.getCheckoutDate1())) {
			if (DateUtil.sdfTime.parse(checkReport.getCheckoutDate1()).getTime() <= DateUtil.sdfsTime
					.parse(checkReport.getCheckinDay() + " " + checkReport.getWorkTime()).getTime()) {
				return true;
			}
		}
		if (StringUtils.isNotBlank(checkReport.getCheckoutDate2())) {
			if (DateUtil.sdfTime.parse(checkReport.getCheckoutDate2()).getTime() <= DateUtil.sdfsTime
					.parse(checkReport.getCheckinDay() + " " + checkReport.getWorkTime()).getTime()) {
				return true;
			}
		}
		return false;
	}

	// 是不是正确的下班时间
	private boolean isOkOffWorkTime(SapCheckReport checkReport) throws Exception {
		if (StringUtils.isBlank(checkReport.getCheckoffException())) {
			return true;
		}
		if (StringUtils.isNotBlank(checkReport.getCheckoutDate1())) {
			if (DateUtil.sdfTime.parse(checkReport.getCheckoutDate1()).getTime() >= DateUtil.sdfsTime
					.parse(checkReport.getCheckinDay() + " " + checkReport.getWorkTime()).getTime()) {
				return true;
			}
		}
		if (StringUtils.isNotBlank(checkReport.getCheckoutDate2())) {
			if (DateUtil.sdfTime.parse(checkReport.getCheckoutDate2()).getTime() >= DateUtil.sdfsTime
					.parse(checkReport.getCheckinDay() + " " + checkReport.getOffWorkTime()).getTime()) {
				return true;
			}
		}
		return false;
	}
}
